import React, {useRef, useState} from 'react';
import {
    UserOutlined,
} from '@ant-design/icons';
import {Layout, Avatar, Tabs, Button} from 'antd';
import Hlexical from './Hlexical';
import ItemTree from "../../components/ItemTree";
import './index.less'
import {useSelector, useDispatch} from "react-redux";
import {addTableBarItem, removeTableBarItem, setActiveKey,editLeftTableOfContents} from "../../redux/tableBarItem_reducer"

const {Sider} = Layout;
const Note = () => {
    const dispatch = useDispatch()
    const [collapsed, setCollapsed] = useState(false);
    //   const {
    //     token: { colorBgContainer },
    //   } = theme.useToken();
    const colorBgContainer = '#800080'
    const newTabIndex = useRef(0);

    const activeKey=useSelector(state => state.tableBarItem.activeKey);
    const items = useSelector(state => state.tableBarItem.data)
    let filePath = useSelector(state => state.dirMessage.dirTree);

    const itemTreeTab=[
        {
            key: '1',
            label: '文件',
            children: <ItemTree filePath={filePath}></ItemTree>,
        },        {
            key: '2',
            label: '标题',
            children: <div id = "leftTableOfContents"></div>,
            forceRender:true
        }
    ]
    const onChange = (newActiveKey) => {
        console.log("setActiveKey(newActiveKey)",newActiveKey)
        dispatch(setActiveKey({"activeKey":newActiveKey}));
    };
    const onChangeLeftTableOfContents = (activeKey)=>{
        dispatch(editLeftTableOfContents({"leftTableOfContents":activeKey}))
    }
    const add = () => {
        const newActiveKey = `newTab${newTabIndex.current++}`;
        dispatch(addTableBarItem(
            {
                label: 'New Tab',
                // children: "",
                key: newActiveKey,
                activeKey:newActiveKey
            }
        ));
    };
    const remove = (targetKey) => {
        console.log("remove = (targetKey):",targetKey)

        let lastIndex = -1;
        items.forEach((item, i) => {
            if (item.key === targetKey) {
                lastIndex = i - 1;
            }
        });
        dispatch(removeTableBarItem(targetKey));
        const newPanes = items.filter((item) => item.key !== targetKey);
        let newActiveKey;
        if (newPanes.length) {
            if (lastIndex >= 0) {
                newActiveKey = newPanes[lastIndex].key;
            } else {
                newActiveKey = newPanes[0].key;
            }
        }
        dispatch(setActiveKey({"activeKey":newActiveKey}));
    };
    const onEdit = (targetKey, action) => {
        if (action === 'add') {
            add();
        } else {
            remove(targetKey);
        }
    };

    return (
        <Layout>
            <Sider margin='5' collapsedWidth={0} collapsed="true">
                <div>
                    <Avatar size={60} src='http://localhost/20231114150555.jpg' icon={<UserOutlined/>}/>
                </div>
                <div>
                    <p style={{color: colorBgContainer, 'fontSize': 60}}>上善若水</p>
                </div>
            </Sider>
            <Sider trigger={null} collapsedWidth={0} width="300px" collapsible collapsed={collapsed}
            //       style={{overflow:"auto"}}
            >
                <Tabs id="itemTreeTabs" defaultActiveKey={useSelector(state => state.tableBarItem.leftTableOfContents)} items={itemTreeTab}
                      onChange={onChangeLeftTableOfContents}
                    style ={{background:"#fff"}}
                />
            </Sider>
            <Layout>
                <Tabs
                    type="editable-card"
                    onChange={onChange}
                    activeKey={activeKey}
                    onEdit={onEdit}
                    items={items.map(item=>{
                        return {label:item.label,children: <div className="HlexicalName"><Hlexical  filePath={item.children}/></div>,key:item.key}
                    })}
                />
            </Layout>
        </Layout>
    );
};
export default Note;
